/*
   Screen routines
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

void screen_center_message( char *msg, int col)
{
 int  x,  y, len;
 int x2, y2, len2;

 len = 8 * strlen( msg) + 8;
 
 x = ( 320 - len) / 2;
 y = 96;

 len2  = 8 * strlen( _(" PRESS ENTER TO CONTINUE ")) + 8;
 x2    = ( 320 - len2) / 2;
 y2    = 140;

 gl_setcontext( spritescreen);
 gl_copyboxfromcontext( physicalscreen,  x - 4, y - 10,  len, 28,  x - 4, y - 10);
 gl_copyboxfromcontext( physicalscreen, x2 - 4, y2 - 4, len2, 16, x2 - 4, y2 - 4);

 gl_setcontext( physicalscreen);

 gl_fillbox( x - 4, y - 10, len, 28, col);
 gl_write( x, y, msg);

 gl_fillbox( x2 - 4, y2 - 4, len2, 16, col);
 gl_write( x2, y2, _(" PRESS ENTER TO CONTINUE "));

 key_wait_for_enter();

 gl_copyboxfromcontext( spritescreen,  x - 4, y - 10,  len, 28,  x - 4, y - 10);
 gl_copyboxfromcontext( spritescreen, x2 - 4, y2 - 4, len2, 16, x2 - 4, y2 - 4);
}

void screen_center_text( char *msg, int y)
{
 int  x, len;

 len = 8 * strlen( msg) + 8;

 x = ( 320 - len) / 2;

 gl_write( x, y, msg);
}


/*
 * Makes a screen shot.
 *
 * Save the screen data (pixel bytes) to a file called:
 *    LINBERTO_SCREENSDIR/linbertossXXXX.raw
 *
 * Where 'XXXX' is the lowest possible number (0000 by default)
 * if that filename exists the 'XXXX' increments by 1 until a
 * non existing filename is found.
 *
 * The palette is saved using the same scheme, but with a '.pal'
 * suffix. The palette is saved in the following format:
 *
 * "Color index, Red, Green, Blue\n"
 *
 * It is not intended for gamers use, it's for my own use,
 * or developers use. For more information see the file
 * 'Screenshot.txt' in the '/doc' directory.
 *
 */
void screen_shot( void)
{
 FILE *fp;
 char fileraw[ 1024];
 char filepal[ 1024];
 char *image;
 int count = 0;
 int colorndx;
 int red, green, blue;

 do {
     sprintf( fileraw, "%s/linbertoss%04d.raw", LINBERTO_SCREENSDIR, count);
     sprintf( filepal, "%s/linbertoss%04d.pal", LINBERTO_SCREENSDIR, count);
     count++;
     if( count > 9999) {
         fprintf( stderr, _("\nToo many screen shots\n\n"));
         return;
     }
 } while( access( fileraw, W_OK) != -1 || 
          access( filepal, W_OK) != -1);

 if( (fp = fopen( fileraw, "w")) == NULL) {
      fprintf( stderr, _("\nCan't save screenshot file: %s: %s\n\n"), fileraw, strerror( errno));
      return;
 }

 /*
  * Allocate a 320 x 200 image buffer
  */
 image = (char *) lin_calloc( 64000);

 /*
  * Now get all the current screen data and put it in 'image'.
  *
  * You could say: "why you didn't use the GraphicsContext 'vbuf'
  * member?"
  * Because it's not good to do this. If the next version of svgalib
  * changes something in GraphicsContext this program could crash.
  * So doing it is the safest way.
  */

 gl_getbox( 0, 0, 320, 200, image);

 fwrite( image, 64000, 1, fp);

 fclose( fp);

 free( image);

 if( (fp = fopen( filepal, "w")) == NULL) {
      fprintf( stderr, _("\nCan't save screenshot file: %s: %s\n\n"), filepal, strerror( errno));
      return;
 }

 for( colorndx = 0; colorndx < 256; colorndx++) {
      vga_getpalette( colorndx, &red, &green, &blue);
      fprintf( fp, "%3d %3d %3d %3d\n", colorndx, red << 2, green << 2, blue << 2);
 }

 fclose( fp);
}

