/* 
   World functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

/* The map array */
char map_stage[ MAP_X_NDX][ MAP_Y_NDX] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
};

/* A temporal array */
char map_tmp[ MAP_X_NDX][ MAP_Y_NDX] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
};

/* The name of the world to edit */
char world_filename[ 4096];

/* The world header for the current open world */
struct world_header swhdr;

/* The directory where the worlds dwell  */
char *dirname;

/* Current stage number */
int curr_stage;

static void world_printmenu( void);
static void world_edit_printmenu( void);

void world_editor( void)
{
 char new_world[ 256];
 char *c;
 int pos = 0;

 while( 1) {
        world_printmenu();

        pos = menu( 100, 138, 112, pos, 3, 3);

        switch( pos) {
                case 0: strcpy( new_world, "------------------------.WLD");
                        if( lin_inputstr( _("Enter the world name"), new_world, 24) > 0) {
                            c = index( new_world, '-');
                            if( c != NULL) *c = '\0';
                            c = index( new_world, '.'); 
                            if( c != NULL) *c = '\0';
                            strcpy( world_filename, new_world);
                            strcat( world_filename, ".wld");
                            dirname = LINBERTO_DATADIR;
                            if( world_filename[ 0] != '.')
                                if( world_check_exist() > 0)
                                    world_edit_menu();
                        }
                        break;
                case 1: world_filename[ 0] = 0;
                        world_select();
                        if( world_filename[ 0] != 0)
                            world_edit_menu();
                        break;
                case 2: /* Do nothing                     */
                        /* Just see the screen refresh :) */
                        break;
                case 3: return;
        }

 } /* end while( 1) */

}

static void world_printmenu( void)
{
 int y;
 int i, j;
 struct timeval tv;

 for( i = 0; i < 319; i++)
      for( j = 0; j < 199; j++) {
           gettimeofday( &tv, (struct timezone *) NULL);
           gl_setpixel( i, j, ((j & i) ^ tv.tv_sec) & 0x000000FF);
      }

 y = 140;

 gl_fillbox(  70, 16, 180, 16, LIN_BLACK);
 screen_center_text( _("World Editor"), 20);

 gl_fillbox(  96, 134, 120, 50, LIN_BLACK);
 screen_center_text( _("New")   , y     );
 screen_center_text( _("Select"), y + 10);
 screen_center_text( _("Void")  , y + 20);
 screen_center_text( _("Return"), y + 30);

 draw_texture_cube( 152, 45, lcolors[ 4]);
 draw_texture_cube( 168, 66, lcolors[ 4]);
 draw_texture_cube( 136, 66, lcolors[ 4]);

 draw_texture_cube( 152, 87, lcolors[ 4]);
 draw_texture_cube( 184, 87, lcolors[ 4]);
 draw_texture_cube( 120, 87, lcolors[ 4]);
}

void world_select( void)
{
 DIR *dp;
 struct dirent *dirp;
 struct stat stbuf;
 char currdir[ 4096]; /* Current working directory             */
 char buf[ 16];       /* Temporal buffer for wildcard matching */
 int stages;
 int len;

 dirname = LINBERTO_DATADIR;

 dp = opendir( dirname);

 if( dp == NULL) {
     fprintf( stderr, _("\nCan't open directory: %s: %s\n\nThe game is not installed. Trying default directory...\n\n"), dirname, strerror( errno));
     dirname = LINBERTO_DATADIR_DEFAULT;
     dp = opendir( dirname);
     if( dp == NULL) {
         fprintf( stderr, _("\nCan't open default directory: %s: %s\n\n"), dirname, strerror( errno));
         exit_game( 1);
     }
 }

 /* Initialize the linked list */
 ll_init();

 gl_clearscreen( 0);

 screen_center_text( _("Searching worlds..."), 40);

 getcwd( currdir, sizeof( currdir) - 1);

 lin_chdir( dirname);

 while(( dirp = readdir( dp)) != NULL ) {
       stat( dirp->d_name, &stbuf);
       if( S_ISREG( stbuf.st_mode)) {
           len = strlen( dirp->d_name);
           if( len < 5)
               continue;

           buf[ 0] = dirp->d_name[ len - 4];
           buf[ 1] = dirp->d_name[ len - 3];
           buf[ 2] = dirp->d_name[ len - 2];
           buf[ 3] = dirp->d_name[ len - 1];
           buf[ 4] = 0;

           /* If it is a world file add it to the linked list */
           if( !strcasecmp( buf, ".wld")) {
               if( check_world_magic( dirp->d_name, &stages) > 0)
                   ll_add( dirp->d_name, stages);
           }
       }

 } /* end while( ...) */

 closedir( dp);

 lin_chdir( currdir);

 if( ll_qty == 0) {
     screen_center_message( _("ERROR! I CAN'T FIND THE WORLDS!"), LIN_RED);
     return;
 }

 gl_clearscreen( 0);

 world_navigate_names();

 ll_free();
}

void world_edit_menu( void)
{
 FILE *fp;
 char filename[ 8192];
 char buf[ 256];
 struct stat stbuf;
 int pos;
 int delstage;

 sprintf( filename, "%s/%s", dirname, world_filename);

 if( ( fp = fopen( filename, "r+")) == NULL) {
       fprintf( stderr, _("\nCan't open world file: %s: %s\n\n"), filename, strerror( errno));
       exit_game( 1);
 }

 fread( &swhdr, sizeof( struct world_header), 1, fp);

 if( strcmp( swhdr.magic, WORLD_MAGIC)) {
     screen_center_message( _(" WORLD FILE INVALID! "), LIN_RED);
     fclose( fp);
     return;
 }

 fstat( fileno( fp), &stbuf);

 fread( &map_stage, sizeof( map_stage), 1, fp);

 curr_stage = pos = 0;

 while( 1) {
        world_edit_printmenu();
        draw_mini_board( 18, 10);

        pos = menu( 168, 8, 110, pos, 7, 7);

        switch( pos) {
                /* Previous stage */
                case 0: if( curr_stage < 1)
                            break;
                        sound_send( SND_BLKN);
                        curr_stage--;
                        fseek( fp, (long) -2 * sizeof( map_stage), SEEK_CUR);
                        fread( &map_stage, sizeof( map_stage), 1, fp);
                        break;
                /* Next stage */
                case 1: if( curr_stage >= swhdr.number - 1)
                            break;
                        sound_send( SND_BLKN);
                        curr_stage++;
                        fread( &map_stage, sizeof( map_stage), 1, fp);
                        break;
                /* Go to the first stage */
                case 2: sound_send( SND_BLKN);
                        fseek( fp, (long) sizeof( struct world_header), SEEK_SET);
                        fread( &map_stage, sizeof( map_stage), 1, fp);
                        curr_stage = 0;
                        break;
                /* Go to the last stage */
                case 3: sound_send( SND_BLKN);
                        fseek( fp, (long) -1 * sizeof( map_stage), SEEK_END);
                        fread( &map_stage, sizeof( map_stage), 1, fp);
                        curr_stage = swhdr.number - 1;
                        break;
                /* Add stage */
                case 4: memcpy( &map_tmp, &map_stage, sizeof( map_stage));
                        memset( &map_stage, -1, sizeof( map_stage));
                        map_stage[ 0][ 7] = 0;
                        if( world_edit() > 0)  {
                            swhdr.number++;
                            fseek( fp, (long) 0, SEEK_SET);
                            fwrite( &swhdr, sizeof( struct world_header), 1, fp);
                            fseek( fp, (long) 0, SEEK_END);
                            fwrite( &map_stage, sizeof( map_stage), 1, fp);
                            fflush( fp);
                            fstat( fileno( fp), &stbuf);
                            curr_stage = swhdr.number - 1;
                            break;
                        }
                        memcpy( &map_stage, &map_tmp, sizeof( map_stage));
                        break;
                /* Delete stage */
                case 5: if( swhdr.number == 1) {
                            screen_center_message( _(" At least 1 stage is needed "), BLUE_LEFT);
                            break;
                        }
                        if( world_delete() > 0) {
                            delstage = curr_stage;
                            while( !feof( fp) && fread( &map_stage, sizeof( map_stage), 1, fp) > 0) {
                                   fseek( fp, (long) -2 * sizeof( map_stage), SEEK_CUR);
                                   fwrite( &map_stage, sizeof( map_stage), 1, fp);
                                   fseek( fp, (long) sizeof( map_stage), SEEK_CUR);
                            }
                            ftruncate( fileno( fp), stbuf.st_size - sizeof( map_stage)); 
                            swhdr.number--;
                            if( curr_stage > swhdr.number - 1)
                                curr_stage = swhdr.number - 1;
                            fseek( fp, (long) 0, SEEK_SET);
                            fwrite( &swhdr, sizeof( struct world_header), 1, fp);
                            fseek( fp, (long) sizeof( struct world_header) + curr_stage * sizeof( map_stage), SEEK_SET);
                            fread( &map_stage, sizeof( map_stage), 1, fp);
                            fflush( fp);
                            fstat( fileno( fp), &stbuf);
                            sprintf( buf, _(" Stage %d deleted "), delstage);
                            screen_center_message( buf, BLUE_LEFT);
                        }
                        break;
                /* Edit stage */
                case 6: memcpy( &map_tmp, &map_stage, sizeof( map_stage));
                        if( world_edit() > 0)  {
                            fseek( fp, (long) -1 * sizeof( map_stage), SEEK_CUR);
                            fwrite( &map_stage, sizeof( map_stage), 1, fp);
                            break;
                        }
                        memcpy( &map_stage, &map_tmp, sizeof( map_stage));
                        break;
                /* Return */
                case 7: fclose( fp);
                        return;
        }

 } /* end while( 1) */

}

void world_navigate_names( void)
{
 screen_center_text( _(" World                          Stages "),   0);
 screen_center_text( _(" Press ENTER to select ")                , 190);
 lin_title_line();

 ll_current     = ll_head;
 ll_browse_curr = ll_head;
 ll_browse_top  = ll_head;
 ll_browse_pos = 0;

 ll_show();

 while( 1) {
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        switch( lin_scancode) {
            case SCANCODE_KEYPADENTER:
            case SCANCODE_ENTER:
                   strcpy( world_filename, ll_browse_curr->filename);
                   return;
            case SCANCODE_Q:
            case SCANCODE_ESCAPE:
                   return;
            case SCANCODE_CURSORLEFT:
            case SCANCODE_CURSORBLOCKLEFT:
            case SCANCODE_CURSORUP:
            case SCANCODE_CURSORBLOCKUP:
                   sound_send( SND_MENU);
                   if( ll_browse_curr->prev == NULL)
                       continue;

                   ll_browse_curr = ll_browse_curr->prev;
                   ll_browse_pos--;
 
                   if( ll_browse_pos < 0) {
                       if( ll_browse_back( 17) == -1) {
                           ll_browse_pos++;
                           ll_show();
                           continue;
                       }
                       ll_browse_pos = 16;
                   }
 
                   ll_show();
                   continue;
            case SCANCODE_CURSORRIGHT:
            case SCANCODE_CURSORBLOCKRIGHT:
            case SCANCODE_CURSORDOWN:
            case SCANCODE_CURSORBLOCKDOWN:
                   sound_send( SND_MENU);
                   if( ll_browse_curr->next == NULL)
                       continue;
 
                   ll_browse_curr = ll_browse_curr->next;
                   ll_browse_pos++;
 
                   if( ll_browse_pos > 16) {
                       if( ll_browse_next( 17) == -1) {
                           ll_browse_pos--;
                           ll_show();
                           continue;
                       }
                      ll_browse_pos = 0;
                   }

                   ll_show();
                   continue;
            case SCANCODE_HOME:
            case SCANCODE_KEYPAD7:
                   sound_send( SND_MENU);
                   ll_browse_top  = ll_head;
                   ll_browse_curr = ll_head;
                   ll_browse_pos  = 0;
                   ll_show();
                   continue;
            case SCANCODE_END:
            case SCANCODE_KEYPAD1:
                   sound_send( SND_MENU);
                   while( ll_browse_next( 17) != -1);
                   ll_browse_curr = ll_browse_top;
                   ll_browse_pos  = 0;
                   ll_show();
                   continue;
            case SCANCODE_PAGEUP:
            case SCANCODE_KEYPAD9:
                   sound_send( SND_MENU);
                   if( ll_browse_back( 17) == -1)
                       continue;
                   ll_browse_pos  = 0;
                   ll_browse_curr = ll_browse_top;
                   ll_show();
                   continue;
            case SCANCODE_PAGEDOWN:
            case SCANCODE_KEYPAD3:
                   sound_send( SND_MENU);
                   if( ll_browse_next( 17) == -1)
                       continue;
                   ll_browse_pos  = 0;
                   ll_browse_curr = ll_browse_top;
                   ll_show();
                   continue;
        } /* end switch */

 } /* end while */

}

static void world_edit_printmenu( void)
{
 gl_clearscreen( 0);

 gl_write( 170, 10, _(" <  Previous "));
 gl_write( 170, 20, _(" >  Next    "));
 gl_write( 170, 30, _(" << First   "));
 gl_write( 170, 40, _(" >> Last   "));
 gl_write( 170, 50, _(" +  Add     "));
 gl_write( 170, 60, _(" -  Delete "));
 gl_write( 170, 70, _(" E  Edit   "));
 gl_write( 170, 80, _(" X  Return "));

 gl_fillbox( 0, 100, 319, 2, LIN_RED);
 gl_fillbox( 0, 135, 319, 2, LIN_RED);
 gl_fillbox( 0, 186, 319, 2, LIN_RED);

 gl_write(  20, 110, _(" Current stage: "));
 grf_printf( 20, 120, "%10ld", curr_stage);

 gl_write(  20, 150, _(" Number of stages: "));
 grf_printf( 20, 160, "%10ld", swhdr.number);
}

int world_edit( void)
{
 int x, y;
 int saved_blockking;  /* Place to save block kind */
 char *image;

 image = (char *) lin_calloc( 1024);

 saved_blockking = opt.cfg_blockkind;

 gl_clearscreen( 0);

 screen_center_text( _("'S' to save   'Q' to quit"),  0);
 screen_center_text( _("ENTER to del/add block")   , 10);

 opt.cfg_blockkind = BLOCK_PLAIN;
 draw_berto_board( 0);

 x = 1;
 y = 0;

before_while:

 gl_getbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);

 opt.cfg_blockkind = BLOCK_WIRE;
 draw_block( BLOCK_X + (x * 16), BLOCK_Y + (y * 21), lcolors[ 1]);

 while( 1) {
       if( keyboard_update() == 0 && joy_moved() == 0)
           continue;

       switch( lin_scancode) {
          case SCANCODE_KEYPADENTER:
          case SCANCODE_ENTER:
                 /* Draw/Erase a block */
                 opt.cfg_blockkind = BLOCK_PLAIN;
                 if( map_stage[ y][ x] == -1) {
                     sound_send( SND_BLKF);
                     map_stage[ y][ x] = 0;
                     draw_block( BLOCK_X + (x * 16), BLOCK_Y + (y * 21), lcolors[ 0]);
                 } else if( map_stage[ y][ x] == 0) {
                            sound_send( SND_BLKD);
                            map_stage[ y][ x] = -1;
                            draw_block( BLOCK_X + (x * 16), BLOCK_Y + (y * 21), lcolors[ 5]);
                 }
                 gl_getbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 opt.cfg_blockkind = BLOCK_WIRE;
                 draw_block( BLOCK_X + (x * 16), BLOCK_Y + (y * 21), lcolors[ 1]);
                 continue;
          case SCANCODE_Q:
          case SCANCODE_ESCAPE:
                 free( image);
                 opt.cfg_blockkind = saved_blockking;
                 return -1;
          case SCANCODE_S:
                 free( image);
                 opt.cfg_blockkind = saved_blockking;
                 return 1;
          case SCANCODE_CURSORLEFT:
          case SCANCODE_CURSORBLOCKLEFT:
          case SCANCODE_KEYPAD1:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x--;
                 y++;
                 if( x < 0 || y > MAP_X_NDX - 1) {
                     x++;
                     y--;
                 }
                 /*
                  * I can't use 'continue' here, because keyboard_update() gets
                  * called too much times, I can't use keyboard_waitforupdate()
                  * because it crash sometimes.
                  */
                 goto before_while;
          case SCANCODE_CURSORRIGHT:
          case SCANCODE_CURSORBLOCKRIGHT:
          case SCANCODE_KEYPAD9:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x++;
                 y--;
                 if( x > MAP_Y_NDX - 1 || y < 0) {
                     x--;
                     y++;
                 }
                 goto before_while;
          case SCANCODE_CURSORUP:
          case SCANCODE_CURSORBLOCKUP:
          case SCANCODE_KEYPAD7:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x--;
                 y--;
                 if( x < 0 || y < 0) {
                     x++;
                     y++;
                 }
                 goto before_while;
          case SCANCODE_CURSORDOWN:
          case SCANCODE_CURSORBLOCKDOWN:
          case SCANCODE_KEYPAD3:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x++;
                 y++;
                 if( x > MAP_Y_NDX - 1 || y > MAP_X_NDX - 1) {
                     x--;
                     y--;
                 }
                 goto before_while;
          case SCANCODE_HOME:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x = 1;
                 y = 0;
                 goto before_while;
          case SCANCODE_END:
                 sound_send( SND_BLKM);
                 gl_putbox( BLOCK_X + (x * 16) - 16, BLOCK_Y + (y * 21), 32, 32, image);
                 x = MAP_Y_NDX - 2;
                 y = MAP_X_NDX - 1;
                 goto before_while;
       } /* end switch */
 } /* end while */

}

int world_delete( void)
{
 char buf[ 256];
 int option;

 gl_clearscreen( 0);

 lin_title_line();

 sprintf( buf, _(" Delete stage %d ? "), curr_stage);
 screen_center_text( buf, 0);

 screen_center_text( _(" Are you sure? "), 20);

 gl_write( 110, 40, _(" Yes "));
 gl_setfontcolors( LIN_WHITE, LIN_BLACK);
 gl_write( 160, 40, _(" No "));
 gl_setfontcolors( LIN_BLACK, LIN_WHITE);

 option = 1;

 while( 1) {
       if( keyboard_update() == 0 && joy_moved() == 0)
           continue;

       switch( lin_scancode) {
          case SCANCODE_KEYPADENTER:
          case SCANCODE_ENTER:
                 if( option)
                     return -1;
                 else
                     return  1;
          case SCANCODE_Q:
          case SCANCODE_ESCAPE:
          case SCANCODE_N:
                 return -1;
          case SCANCODE_Y:
          case SCANCODE_S:
                 return 1;
          case SCANCODE_CURSORLEFT:
          case SCANCODE_CURSORBLOCKLEFT:
          case SCANCODE_CURSORUP:
          case SCANCODE_CURSORBLOCKUP:
          case SCANCODE_CURSORRIGHT:
          case SCANCODE_CURSORBLOCKRIGHT:
          case SCANCODE_CURSORDOWN:
          case SCANCODE_CURSORBLOCKDOWN:
          case SCANCODE_HOME:
          case SCANCODE_KEYPAD7:
          case SCANCODE_END:
          case SCANCODE_KEYPAD1:
                 sound_send( SND_MENU);
                 option = option ? 0: 1;

                 if( option) 
                     gl_setfontcolors( LIN_BLACK, LIN_WHITE);
                 else
                     gl_setfontcolors( LIN_WHITE, LIN_BLACK);

                 gl_write( 110, 40, _(" Yes "));

                 if( option)
                     gl_setfontcolors( LIN_WHITE, LIN_BLACK);
                 else
                     gl_setfontcolors( LIN_BLACK, LIN_WHITE);

                 gl_write( 160, 40, _(" No "));

                 gl_setfontcolors( LIN_BLACK, LIN_WHITE);
                 continue;
       } /* end switch */
 } /* end while */

 return -1;
}

int world_check_exist( void)
{
 char filename[ 8192];
 struct world_header whdr;
 int fd;

 sprintf( filename, "%s/%s", dirname, world_filename);

 if( access( filename, F_OK) == 0) {
     screen_center_message( _(" This world already exists "), LIN_RED);
     return -1;
 }

 fd = creat( filename, 0600);

 if( fd == -1) {
     fprintf( stderr, _("\nCan't create new world file: %s: %s\n\n"), filename, strerror( errno));
     exit_game( 1);
 }

 memset( &whdr, 0, sizeof( struct world_header));

 strcpy( whdr.magic, WORLD_MAGIC);
 whdr.number = 1;

 memset( &map_stage, -1, sizeof( map_stage));
 map_stage[ 0][ 7] = 0;
 
 write( fd, &whdr     , sizeof( struct world_header));
 write( fd, &map_stage, sizeof( map_stage          ));

 close( fd);

 return 1;
}

